package bcn.coord;

import bcn.InputPort;
import bcn.InputRegister;
import bcn.Behavior;
import bcn.OutputPort;


public class PriorityCoord<T> extends Behavior {
    
    protected final InputPort<T>[] _in;
    protected final OutputPort<T> _out;
    protected final OutputPort<Integer> _winner;
    protected int lastWinner;
    protected T msgWinner;
    
    
    public PriorityCoord(int count) {
    	if (count < 2) {
    		throw new IllegalArgumentException("A coordinator with less than 2 inputs doesn't make sense!");
    	}
        _in = new InputPort[count];
        for (int i=0; i<count; i++) {
            _in[i] = new InputRegister<T>();
        }
        _out = new OutputPort<T>();
        _winner = new OutputPort<Integer>();
        lastWinner = Integer.MAX_VALUE;
    }
    
    
    public final InputPort<T> get_in(int id) {
        return _in[id];
    }
    
    
    public final OutputPort<T> get_out() {
        return _out;
    }
    
    
    public final OutputPort<Integer> get_winner() {
        return _winner;
    }
    
    
    @Override
    public void step() {
    	/* The behavior with the highest priority takes control for two control steps:
    	 * messages are removed from the input registers;
    	 * if a behavior doesn't need to take control it doesn't send messages;
    	 * if a behavior wants to take control it must send messages at each control step. */
    	for (int i=0; i<_in.length; i++) {
    	    T msg = _in[i].get();
    	    if (msg != null) {
                lastWinner = i;
                msgWinner = msg;
                _out.send(msg);
                _winner.send(i);
                //System.out.println("PriorityCoord: " + i + " wins");
                // flush other inputs
                for (i++; i<_in.length; i++) {
                    _in[i].get();
                }
                return;
    	    } else if (i >= lastWinner) {  // last winner takes control for another control step
                _out.send(msgWinner);
                _winner.send(lastWinner);
                //System.out.println("PriorityCoord: " + lastWinner + " won last control step");
                lastWinner = Integer.MAX_VALUE;
                return;
    	    }
    	}
    	// no inputs received for two control steps
        _winner.send(-1);
    }
    
}
