package foraging;


public class Wander extends foraging_stub.Wander_stub {
    
    private final FSM fsm;
    private final int id;
    
    
    public Wander(int coordID) {
        super();
        fsm = new FSM(FORWARD, TURN);
        id = coordID;
    }
    
    
    @Override
    public void init() {
        fsm.init();
    }
    
    
    @Override
    public void step() {
        Integer w = _winner.peek();
        if (w != null  &&  w != id) {  // the behavior has been suppressed
            fsm.reset();
            //System.out.println("Wander: reset");
        } else {
            fsm.exec();
        }
    }
    
    
    private final FSM.State FORWARD = new FSM.State() {
        private int wait;
        
        @Override
        protected void entry() {
            wait = Misc.steps(100*Misc.rndRange(10, 51));  // [1,5] seconds
            _drive.send(new Drive.Speed(1, 1));
        }
        
        @Override
        protected void exec() {
            wait--;
            if (wait <= 0) {
                transition(TURN);
            } else {
                _drive.send(new Drive.Speed(1, 1));
            }
        }
    };
    
    
    private final FSM.State TURN = new FSM.State() {
        private boolean left;
        private int wait;
        
        @Override
        protected void entry() {
            wait = Misc.steps(100*Misc.rndRange(3, 16));  // [0.3,1.5] seconds
            left = Misc.RANDOM.nextBoolean();
            _drive.send(new Drive.Speed(left ? 0 : 1, left ? 1 : 0));
        }
        
        @Override
        protected void exec() {
            wait--;
            if (wait <= 0) {
                transition(FORWARD);
            } else {
                _drive.send(new Drive.Speed(left ? 0 : 1, left ? 1 : 0));
            }
        }
    };
    
}
