package foraging;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;

import bcn.InputPort;


public class WinnerInspector extends JFrame {
    
    private class Panel extends JPanel {
        private final Color GOLD = new Color(0xFDD017);
        
        @Override
        protected void paintComponent(Graphics gr) {
            super.paintComponent(gr);
            Graphics2D g = (Graphics2D)gr;
            g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                               RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            int w = getWidth(), h = getHeight();
            float y, dy = ((float)h)/behaviors.length;
            gr.setFont(gr.getFont().deriveFont(Font.BOLD, 13));
            FontMetrics fm = gr.getFontMetrics();
            for (int i=0; i<behaviors.length; i++) {
                y = i*dy;
                if (i == winner) {
                    gr.setColor(GOLD);
                    gr.fillRect(0, (int)y, w, (int)dy);
                } else {
                    gr.setColor(Color.LIGHT_GRAY);
                    gr.fillRect(0, (int)y, w, (int)dy);
                }
                gr.setColor(Color.BLACK);
                gr.drawString(behaviors[i],
                              (w-fm.stringWidth(behaviors[i]))/2,
                              (int)(y + dy/2 + fm.getAscent()/2));
                if (i != 0) {
                    gr.drawLine(0, (int)y, w, (int)y);
                }
            }
        }
    }
    
    private String[] behaviors;
    private int winner;
    private JPanel pn;
    
    
    public WinnerInspector(String name, String... behaviors) {
        super(name);
        this.behaviors = behaviors;
        this.winner = -1;
        this.pn = new Panel();
        this.add(pn);
        this.setSize(140, behaviors.length*25);
        this.setResizable(false);
        this.setAlwaysOnTop(true);
        this.setVisible(true);
    }
    
    
    public InputPort<Integer> getInputPort () {
        return new InputPort<Integer>() {
            @Override
            public void put(Integer msg) {
                winner = msg;
                pn.repaint();
            }
            @Override
            public Integer get() {
                return winner;
            }
            @Override
            public Integer peek() {
                return winner;
            }
        };
    }
    
}
